/*
	TwoDViewer.class				Petri Dish Sim 1.0 release
	By: Jonathan Zepp - @DaJMasta
	
	Holds a simple UI for visualizing the world, adding components, and reading information from individual cells.  The UI is simple, not the totality of the class...
	
	July 30, 2015
	
	
	This code and program can be used according the the GPL v3 found here: http://www.gnu.org/licenses/gpl-3.0.en.html
	Free to modify and use in your own projects, but not for use in any commercial product.  If you use it in your project, I'd appreciate a credit as a source!
*/

package WorldSim;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DecimalFormat;
import java.util.Date;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class TwoDViewer extends JPanel implements ActionListener{
	Color[][] heatMap ;
	Color[][] lightMap ;
	Color[][] nutrientMap ;
	Color[][] entityMap ;
	Color[][] actionMap ;
	Color[][] subconsciousActionMap ;
	int zoomScale = 7 ;
	World world ;
	WorldEngine worldEngine ;
	long graphicsFrameStart,lastGraphicsFrame ;
	int worldXSize, worldYSize, worldZSize, visualizationZPosition, maxLight, minLight, viewerX, viewerY, viewerZ ;
	double heatAutoGamma, lightAutoGamma ;
	volatile boolean bSimActive ;
	int simStepDelay = 100 ;
	TextField stepField ;
	boolean bLargeWorld = false ;
	boolean bSpawnToggle ;
	Entity entityPanelViewed ;
	JFrame application, controls, entityViewer ;
	
	
	double foodEnergyScale = 10.0 ;
	JCheckBox bigWorldBox, organismsDriftBox, allowBasicBox ;
	JButton resetButton, goUpButton, goDownButton, entityViewXUp, entityViewXDown, entityViewYUp, entityViewYDown, entityViewZUp, entityViewZDown, entityViewGoButton, setFacingButton, autoSpawnButton ;
	TextField immediateQty, spawnerQty, facingHeat, facingLight, entityViewX, entityViewY, entityViewZ ;
	JComboBox<String[]> immediateAdd, spawnerAdd, resetTemplate, depthSelector, driftSelector, addFromSelector, environmentSelector, facingSelector ;
	JLabel entityViewXLabel, entityViewYLabel, entityViewZLabel, entityViewType, entityViewName, entityViewHeat, entityViewLight, entityViewEnergy, entityViewNutrients, 
			entityViewAction, entityViewSubAction, entityViewLifetime, entityViewHP, entityViewProgeny, entityViewAttacker, entityViewSignalRecieved, entityViewAbilities, entityViewAbilities2 ;
	
	public TwoDViewer(JFrame mainApp, JFrame controlFrame, JFrame viewerFrame)                      				//Mostly setting up UI components for the three windows
	    {
			super();	        
			application = mainApp ;
			controls = controlFrame ;
			entityViewer = viewerFrame ;
			
	        String[] addNowChoices = {"Light", "Variable Light", "Heat Source", "Variable Heat Source", "Food", "Nutrient", "Basic Organism", "Advanced Organism", "Cyanobacteria", "Nitrogen Fixer", "Phytoplankton", "Super Organism", "Zooplankton" } ;
	        String[] addSpawnerChoices = {"Food", "Nutrient"} ;
	        String[] resetChoices = {"Nothing", "Low", "Medium", "High", "Bountiful"} ;
	        String[] depthChoices = {"1", "3", "5", "10"} ;
	        String[] driftChoices = {"None", "Low", "High", "Gravity", "+X current", "+Y current"} ;
	        String[] addFromChoices = {"Anywhere", "Top", "Random sphere"} ;
	        String[] environmentChoices = {"Normal", "Hot", "Cold", "Very cold", "More sources"} ;
	        String[] facingChoices = {"+X side", "-X side", "+Y side", "-Y side", "+Z side", "-Z side"} ;
	        
	        JButton stepForwardButton = new JButton("Simulation Step") ;
	        resetButton = new JButton("Reset World") ;
	        immediateAdd = new JComboBox(addNowChoices) ;
	        spawnerAdd = new JComboBox(addSpawnerChoices) ;
	        resetTemplate = new JComboBox(resetChoices) ;
	        JButton addNowButton = new JButton("Add to world") ;
	        JButton addSpawnerButton = new JButton("Spawn per step") ;
	        JButton setDriftButton = new JButton("Set drift") ;
	        JButton setSpawnButton = new JButton("Set spawn location") ;
	        
	        goUpButton = new JButton("Up") ;
	        goDownButton = new JButton("Down") ;
	        JLabel worldDepth = new JLabel("World Depth:") ;
	        depthSelector = new JComboBox(depthChoices) ;
	        JLabel heightViewer = new JLabel("View height:") ;
	        
	        facingSelector = new JComboBox(facingChoices) ;
	        JLabel facingHeatLabel = new JLabel("Heat from world edge") ;
	        JLabel facingLightLabel = new JLabel("Light from world edge") ;
	        facingHeat = new TextField("1", 4) ;
	        facingLight = new TextField("0", 4) ;
	        setFacingButton = new JButton("Apply") ;
	        
	    	JButton simStartButton = new JButton("Start sim") ;
	    	JButton simStopButton = new JButton("Stop sim") ;
	    	JButton applyAmbientButton = new JButton("Set environment") ;
	    	stepField = new TextField("100", 4) ;
	    	immediateQty = new TextField("1", 4) ;
	    	spawnerQty = new TextField("1", 4) ;

	    	JLabel simDelayLabel = new JLabel("Frame delay (ms):") ;
	    	JLabel worldDropDown = new JLabel("Resource preset:") ;
	    	
	    	bigWorldBox = new JCheckBox("Large world") ;
	    	
	    	organismsDriftBox = new JCheckBox("Organisms can drift") ;
	    	autoSpawnButton = new JButton("Start auto-populate") ;
	    	allowBasicBox = new JCheckBox("Auto populate basic organisms") ;
	    	driftSelector = new JComboBox(driftChoices) ;
	    	addFromSelector = new JComboBox(addFromChoices) ;
	    	environmentSelector = new JComboBox(environmentChoices) ;
	    	JLabel environmentLabel = new JLabel("Environment settings") ;
	    	JLabel addFromLabel = new JLabel("Food spawn area") ;
	    	JLabel driftLabel = new JLabel("Drift/current settings") ;
	    	
	    	entityViewXLabel = new JLabel("X:") ;
	    	entityViewYLabel = new JLabel("Y:") ;
	    	entityViewZLabel = new JLabel("Z:") ;
	    	entityViewType = new JLabel("Entity") ;
	    	entityViewName = new JLabel("This Entity") ;
	    	entityViewHeat = new JLabel("Heat: ") ;
	    	entityViewLight = new JLabel("Illumination: ") ;
	    	entityViewEnergy = new JLabel("Energy: 0, Toxicity: 0") ;
	    	entityViewNutrients = new JLabel("Nutrients - Red: 0, Green: 0, Blue: 0") ;
	    	entityViewAction = new JLabel("No Action") ;
	    	entityViewLifetime = new JLabel("Lifetime: 0") ;
	    	entityViewSubAction = new JLabel("No Action") ;
	    	entityViewHP = new JLabel("Health: 0") ;
	    	entityViewXUp = new JButton("X+") ;
	    	entityViewXDown = new JButton("X-") ;
	    	entityViewYUp = new JButton("Y+") ;
	    	entityViewYDown = new JButton("Y-") ;
	    	entityViewZUp = new JButton("Z+") ;
	    	entityViewZDown = new JButton("Z-") ;
	    	entityViewGoButton = new JButton("Go") ;
	    	entityViewX = new TextField("0", 3) ;
	    	entityViewY = new TextField("0", 3) ;
	    	entityViewZ = new TextField("0", 3) ;
	    	entityViewProgeny = new JLabel("No children") ;
	    	entityViewAttacker = new JLabel("Not attacked") ;
	    	entityViewSignalRecieved = new JLabel("Not signaled") ;
	    	entityViewAbilities = new JLabel("No abilities") ;
	    	entityViewAbilities2 = new JLabel("") ;
	    	
	        setBackground(Color.DARK_GRAY);
	        graphicsFrameStart = 0 ;
	        lastGraphicsFrame = 0 ;
	        worldXSize = 50 ;
	        worldYSize = 50 ;
	        worldZSize = 1 ;
	        visualizationZPosition = 0 ;
	        bSimActive = false ;
	        
	        viewerX = 0 ;
	        viewerY = 0 ;
	        viewerZ = 0 ;
	        
	        heatAutoGamma = 4.0 ;
	        lightAutoGamma = 4.5 ;
	        maxLight = 1 ;
	        minLight = 9999 ;
	        
	        bSpawnToggle = false ;
	        
	        resetWorld() ;
	        
	        application.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); 
	        controls.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);      
	        entityViewer.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);    

	        application.setSize(1280, 1010);         
	        application.setVisible(true);
	        
	        controls.setSize(600, 520);         
	        controls.setVisible(true);
	        controls.setLayout(null) ;
	        
	        entityViewer.setSize(500, 300);         
	        entityViewer.setVisible(true);
	        entityViewer.setLayout(null) ;
	        
	        stepForwardButton.setBounds(20, 20, 170, 35) ;
	        stepForwardButton.setActionCommand("STEP") ;
	        stepForwardButton.addActionListener(this) ;
	        autoSpawnButton.setBounds(20, 65, 170, 35) ;
	        autoSpawnButton.setActionCommand("TOGGLESPAWN") ;
	        autoSpawnButton.addActionListener(this) ;
	        allowBasicBox.setBounds(205, 75, 220, 20) ;
	        allowBasicBox.setSelected(true) ;

	        worldDepth.setBounds(395, 315, 100, 20) ;
	        depthSelector.setBounds(500, 315, 50, 20) ;
	        resetButton.setBounds(440, 410, 130, 60) ;
	        resetButton.setActionCommand("RESET") ;
	        resetButton.addActionListener(this) ;
	        bigWorldBox.setBounds(460, 380, 120, 20) ;
	        resetTemplate.setBounds(475, 350, 90, 20);
	        worldDropDown.setBounds(365, 350, 120, 20);
	        
	        simStartButton.setBounds(230, 15, 100, 50) ;
	        simStartButton.setActionCommand("START") ;
	        simStartButton.addActionListener(this) ;
	        simStopButton.setBounds(340, 15, 100, 50) ;
	        simStopButton.setActionCommand("STOP") ;
	        simStopButton.addActionListener(this) ;
	        stepField.setBounds(485, 40, 50, 20) ;
	        simDelayLabel.setBounds(455, 20, 150, 15);
	        
	        immediateQty.setBounds(145, 120, 50, 20) ;
	        immediateAdd.setBounds(210, 120, 160, 20) ;
	        addNowButton.setBounds(385, 110, 180, 40) ;
	        addNowButton.setActionCommand("ADDNOW") ;
	        addNowButton.addActionListener(this) ;
	        
	        spawnerQty.setBounds(145, 180, 50, 20) ;
	        spawnerAdd.setBounds(210, 180, 160, 20) ;
	        addSpawnerButton.setBounds(385, 170, 180, 40) ;
	        addSpawnerButton.setActionCommand("SPAWN") ;
	        addSpawnerButton.addActionListener(this) ;
	        
	        goUpButton.setBounds(25, 145, 80, 40) ;
	        goDownButton.setBounds(25, 195, 80, 40) ;
	        goUpButton.setActionCommand("UP") ;
	        goUpButton.addActionListener(this) ;
	        goDownButton.setActionCommand("DOWN") ;
	        goDownButton.addActionListener(this) ;
	        heightViewer.setBounds(30, 120, 110, 20) ;
	        
	        organismsDriftBox.setBounds(45, 385, 140, 20) ;
	        driftSelector.setBounds(55, 360, 100, 20) ;
	        driftLabel.setBounds(45, 335, 140, 20) ;
	        setDriftButton.setBounds(210, 350, 100, 40) ;
	        setDriftButton.setActionCommand("DRIFT") ;
	        setDriftButton.addActionListener(this) ;
	        
	        addFromSelector.setBounds(230, 240, 140, 20) ;
	        addFromLabel.setBounds(125, 240, 140, 20) ;
	        setSpawnButton.setBounds(385, 230, 180, 40) ;
	        setSpawnButton.setActionCommand("SPAWNLOCATION") ;
	        setSpawnButton.addActionListener(this) ;
	        
	        environmentSelector.setBounds(50, 300, 110, 20) ;
	        environmentLabel.setBounds(45, 275, 140, 20) ;
	        applyAmbientButton.setBounds(190, 280, 130, 40) ;
	        applyAmbientButton.setActionCommand("ENVIRON") ;
	        applyAmbientButton.addActionListener(this) ;
	        
	        facingSelector.setBounds(230, 435, 80, 20) ;
	        facingHeatLabel.setBounds(20, 425, 140, 20) ;
	        facingLightLabel.setBounds(20, 445, 140, 20) ;
	        facingHeat.setBounds(160, 425, 60, 20) ;
	        facingLight.setBounds(160, 445, 60, 20) ;
	        setFacingButton.setBounds(320, 425, 80, 40) ;
	        setFacingButton.setActionCommand("FACING") ;
	        setFacingButton.addActionListener(this) ; 
	        
	        entityViewXUp.setBounds(110, 100, 50, 50) ;
	        entityViewXDown.setBounds(10, 100, 50, 50) ;
	        entityViewYUp.setBounds(60, 150, 50, 50) ;
	        entityViewYDown.setBounds(60, 50, 50, 50) ;
	        entityViewZUp.setBounds(90, 205, 50, 50) ;
	        entityViewZDown.setBounds(30, 205, 50, 50) ;
	        entityViewXUp.setActionCommand("XUP") ;
	        entityViewXUp.addActionListener(this) ;
	        entityViewXDown.setActionCommand("XDOWN") ;
	        entityViewXDown.addActionListener(this) ;
	        entityViewYUp.setActionCommand("YUP") ;
	        entityViewYUp.addActionListener(this) ;
	        entityViewYDown.setActionCommand("YDOWN") ;
	        entityViewYDown.addActionListener(this) ;
	        entityViewZUp.setActionCommand("ZUP") ;
	        entityViewZUp.addActionListener(this) ;
	        entityViewZDown.setActionCommand("ZDOWN") ;
	        entityViewZDown.addActionListener(this) ;
	        entityViewGoButton.setBounds(150, 5, 50, 40) ;
	        entityViewGoButton.setActionCommand("GO");
	        entityViewGoButton.addActionListener(this) ;
	        entityViewXLabel.setBounds(10, 15, 15, 20) ;
	        entityViewYLabel.setBounds(55, 15, 15, 20) ;
	        entityViewZLabel.setBounds(100, 15, 15, 20) ;
	        entityViewX.setBounds(25, 15, 30, 20) ;
	        entityViewY.setBounds(70, 15, 30, 20) ;
	        entityViewZ.setBounds(115, 15, 30, 20) ;
	        entityViewType.setBounds(245, 10, 300, 20) ;
	        entityViewName.setBounds(215, 35, 300, 20) ;
	        entityViewLifetime.setBounds(375, 35, 300, 20) ;
	        entityViewHeat.setBounds(215, 55, 300, 20) ;
	        entityViewLight.setBounds(360, 55, 300, 20) ;
	        entityViewEnergy.setBounds(215, 75, 300, 20) ;
	        entityViewNutrients.setBounds(215, 95, 300, 20) ;
	        entityViewHP.setBounds(215, 115, 300, 20) ;
	        entityViewProgeny.setBounds(355, 115, 300, 20) ;
	        entityViewAction.setBounds(215, 135, 300, 20) ;
	        entityViewSubAction.setBounds(215, 155, 300, 20) ;
	        entityViewAttacker.setBounds(215, 175, 300, 20) ;
	        entityViewSignalRecieved.setBounds(215, 195, 300, 20) ;
	        entityViewAbilities.setBounds(215, 215, 300, 20) ;
	        entityViewAbilities2.setBounds(215, 235, 300, 20);
	        
	        entityViewer.add(entityViewXUp) ;
	        entityViewer.add(entityViewXDown) ;
	        entityViewer.add(entityViewYUp) ;
	        entityViewer.add(entityViewYDown) ;
	        entityViewer.add(entityViewZUp) ;
	        entityViewer.add(entityViewZDown) ;
	        entityViewer.add(entityViewXUp) ;
	        entityViewer.add(entityViewX) ;
	        entityViewer.add(entityViewY) ;
	        entityViewer.add(entityViewZ) ;
	        entityViewer.add(entityViewXLabel) ;
	        entityViewer.add(entityViewYLabel) ;
	        entityViewer.add(entityViewZLabel) ;
	        entityViewer.add(entityViewGoButton) ;
	        entityViewer.add(entityViewType) ;
	        entityViewer.add(entityViewName) ;
	        entityViewer.add(entityViewHeat) ;
	        entityViewer.add(entityViewLight) ;
	        entityViewer.add(entityViewEnergy) ;
	        entityViewer.add(entityViewNutrients) ;
	        entityViewer.add(entityViewAction) ;
	        entityViewer.add(entityViewLifetime) ;
	        entityViewer.add(entityViewHP) ;
	        entityViewer.add(entityViewSubAction) ;
	        entityViewer.add(entityViewProgeny) ;
	        entityViewer.add(entityViewAttacker) ;
	        entityViewer.add(entityViewSignalRecieved) ;
	        entityViewer.add(entityViewAbilities) ;
	        entityViewer.add(entityViewAbilities2) ;
	        
	        controls.add(stepForwardButton) ;
	        controls.add(resetButton) ;
	        controls.add(simStartButton) ;
	        controls.add(simStopButton) ;
	        controls.add(stepField) ;
	        controls.add(simDelayLabel) ;
	        controls.add(applyAmbientButton) ;
	        controls.add(bigWorldBox) ;
	        controls.add(resetTemplate);
	        controls.add(worldDropDown);
	        controls.add(immediateQty) ;
	        controls.add(spawnerQty) ;
	        controls.add(immediateAdd) ;
	        controls.add(spawnerAdd) ;
	        controls.add(addNowButton);
	        controls.add(addSpawnerButton);
	        controls.add(goUpButton) ;
	        controls.add(goDownButton); 
	        controls.add(worldDepth) ;
	        controls.add(depthSelector) ;
	        controls.add(organismsDriftBox) ;
	        controls.add(driftSelector) ;
	        controls.add(setDriftButton) ;
	        controls.add(addFromSelector) ;
	        controls.add(setSpawnButton) ;
	        controls.add(environmentSelector) ;
	        controls.add(autoSpawnButton) ;
	        controls.add(environmentLabel) ;
	        controls.add(addFromLabel) ;
	        controls.add(driftLabel) ;
	        controls.add(facingSelector) ;
	        controls.add(facingHeatLabel) ;
	        controls.add(facingLightLabel) ;
	        controls.add(facingHeat) ;
	        controls.add(facingLight) ;
	        controls.add(setFacingButton) ;
	        controls.add(allowBasicBox) ;
	        controls.add(heightViewer) ;
	        
	        entityPanelRead() ;
	    }

public void paintComponent(Graphics g)																		  // draw graphics in the visualization panel
	    {
	        int tempX, tempY ;
	        DecimalFormat df = new DecimalFormat("#.##") ;
	        
	        graphicsFrameStart = new Date().getTime() ;

	        super.paintComponent(g);            
	        
	        autoGamma() ;
	        
	        tempX = 15 ;
	        tempY = 30 ;
	        
	        g.setColor(Color.WHITE) ; 
	        g.drawString("Heat map:", tempX + (worldXSize * (zoomScale + 1)) / 2 - 40, tempY - 12) ;
	        
	        for(int x = 0; x < worldXSize; x++){
	        	for(int y = 0; y < worldYSize; y++){
	        		if(x == viewerX && y == viewerY){
	        			if(visualizationZPosition == viewerZ)
	        				g.setColor(Color.WHITE) ;
	        			else
	        				g.setColor(Color.GRAY) ;
	        			g.fillRect((tempX + x * (zoomScale + 1)) - 1, (tempY + y * (zoomScale + 1)) - 1, zoomScale + 2, zoomScale + 2) ;
	        		}
	        		g.setColor(gammaUp(heatMap[x][y], heatAutoGamma)) ;
	    	        g.fillRect((tempX + x * (zoomScale + 1)), (tempY + y * (zoomScale + 1)), zoomScale, zoomScale);
	        	}
	        }
	        
	        tempX += 15 + worldXSize * (zoomScale + 1) ;
	        tempY = 30 ;
	        
	        g.setColor(Color.WHITE) ; 
	        g.drawString("Light map:", tempX + (worldXSize * (zoomScale + 1)) / 2 - 40, tempY - 12) ;
	        
	        for(int x = 0; x < worldXSize; x++){
	        	for(int y = 0; y < worldYSize; y++){
	        		if(x == viewerX && y == viewerY){
	        			if(visualizationZPosition == viewerZ)
	        				g.setColor(Color.WHITE) ;
	        			else
	        				g.setColor(Color.GRAY) ;
	        			g.fillRect((tempX + x * (zoomScale + 1)) - 1, (tempY + y * (zoomScale + 1)) - 1, zoomScale + 2, zoomScale + 2) ;
	        		}
	        		g.setColor(gammaUp(lightMap[x][y], lightAutoGamma)) ;
	    	        g.fillRect((tempX + x * (zoomScale + 1)), (tempY + y * (zoomScale + 1)), zoomScale, zoomScale) ;
	        	}
	        }
	        
	        tempX += 15 + worldXSize * (zoomScale + 1) ;
	        tempY = 30 ;
	        
	        g.setColor(Color.WHITE) ;
	        g.drawString("Nutrient map:", tempX + (worldXSize * (zoomScale + 1)) / 2 - 40, tempY - 12) ;
	        
	        for(int x = 0; x < worldXSize; x++){
	        	for(int y = 0; y < worldYSize; y++){
	        		if(x == viewerX && y == viewerY){
	        			if(visualizationZPosition == viewerZ)
	        				g.setColor(Color.WHITE) ;
	        			else
	        				g.setColor(Color.GRAY) ;
	        			g.fillRect((tempX + x * (zoomScale + 1)) - 1, (tempY + y * (zoomScale + 1)) - 1, zoomScale + 2, zoomScale + 2) ;
	        		}
	        		g.setColor(gammaUp(nutrientMap[x][y], 2.5)) ;
	    	        g.fillRect((tempX + x * (zoomScale + 1)), (tempY + y * (zoomScale + 1)), zoomScale, zoomScale) ;
	        	}
	        }
	        
	        tempX = 30 + worldXSize * (zoomScale + 1) ;
	        tempY = 65 + worldXSize * (zoomScale + 1) ;
	        
	        g.setColor(Color.WHITE) ;
	        g.drawString("Entity map:", tempX + (worldXSize * (zoomScale + 1)) / 2 - 35, tempY - 12) ;
	        
	        for(int x = 0; x < worldXSize; x++){
	        	for(int y = 0; y < worldYSize; y++){
	        		if(x == viewerX && y == viewerY){
	        			if(visualizationZPosition == viewerZ)
	        				g.setColor(Color.WHITE) ;
	        			else
	        				g.setColor(Color.GRAY) ;
	        			g.fillRect((tempX + x * (zoomScale + 1)) - 1, (tempY + y * (zoomScale + 1)) - 1, zoomScale + 2, zoomScale + 2) ;
	        		}
	        		g.setColor(entityMap[x][y]) ;
	    	        g.fillRect((tempX + x * (zoomScale + 1)), (tempY + y * (zoomScale + 1)), zoomScale, zoomScale) ;
	        	}
	        }
	        
	        tempX += 15 + worldXSize * (zoomScale + 1) ;
	        tempY = 65 + worldXSize * (zoomScale + 1) ;
	        
	        g.setColor(Color.WHITE) ;
	        g.drawString("Organism action map:", tempX + (worldXSize * (zoomScale + 1)) / 2 - 60, tempY - 12) ;
	        
	        for(int x = 0; x < worldXSize; x++){
	        	for(int y = 0; y < worldYSize; y++){
	        		if(x == viewerX && y == viewerY){
	        			if(visualizationZPosition == viewerZ)
	        				g.setColor(Color.WHITE) ;
	        			else
	        				g.setColor(Color.GRAY) ;
	        			g.fillRect((tempX + x * (zoomScale + 1)) - 1, (tempY + y * (zoomScale + 1)) - 1, zoomScale + 2, zoomScale + 2) ;
	        		}
	        		g.setColor(actionMap[x][y]) ;
	    	        g.fillRect((tempX + x * (zoomScale + 1)), (tempY + y * (zoomScale + 1)), zoomScale, zoomScale) ;
	    	        if(subconsciousActionMap[x][y] != Color.BLACK){
	    	        	g.setColor(subconsciousActionMap[x][y]) ;
		    	        g.fillRect((tempX + (x * (zoomScale + 1)) + (int)((zoomScale + 1) / 4)), (tempY + (y * (zoomScale + 1)) + (int)((zoomScale + 1) / 4)), (int)((zoomScale - 1) / 2), (int)((zoomScale - 1) / 2)) ;
	    	        }
	        	}
	        }
	        
	        g.setColor(Color.WHITE) ;																				//The text stats
	        
	        tempX = 25 ;
	        tempY = 55 + worldXSize * (zoomScale + 1) ;
	        
	        g.drawString("Looking at height: " + (int)visualizationZPosition, tempX + 120, tempY) ;
	        tempY += 25 ;
	        g.drawString("World Stats: ", tempX + 10, tempY) ;
	        tempY += 22 ;
	        g.drawString("Simulation step: " + (int)world.age, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Steps per second: " + worldEngine.fpsCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Graphics update time: " + lastGraphicsFrame + "ms", tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("World step time: " + worldEngine.lastWorldFrame + "ms", tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Delay between frames: " + (int)simStepDelay + "ms", tempX, tempY) ;
	        tempY += 17 ;
	        if(bLargeWorld)
	        	g.drawString("World size: 100 x 100 x " + (int)worldZSize, tempX, tempY) ;
	        else
	        	g.drawString("World size: 50 x 50 x " + (int)worldZSize, tempX, tempY) ;    
	        tempY += 17 ;
	        g.drawString("Light count: " + (int)world.lastTotalLights, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total light output: " + (int)world.lastIlluminationTotal, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Max illumination: " + (int)worldEngine.stepMaxLight, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total heat: " + df.format(worldEngine.totalHeat), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Max heat: " + df.format(worldEngine.stepMaxHeat), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Min heat: " + df.format(worldEngine.stepMinHeat), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Ambient heat: " + df.format((world.wallTemps[0] + world.wallTemps[1] + world.wallTemps[2] + world.wallTemps[3] + world.wallTemps[4] + world.wallTemps[5]) / 6.0), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Random drift chance: " + world.driftVarianceX + "%", tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Current - X: " + world.driftSpeedX + ", Y: " + world.driftSpeedY + ", Z: " + world.driftSpeedZ, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Food per step: " + (int)(worldEngine.getSpawner(new RandomFood(world.heatFloor))).quantity, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Nutrient sources per step: " + (int)(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).quantity, tempX, tempY) ;
	        tempY += 17 ;
	        if(bSpawnToggle)
	        	g.drawString("Organisms randomly added", tempX, tempY) ;
	        else
	        	g.drawString("No active organism adding", tempX, tempY) ;
	        tempY += 17 ;
	        if(worldEngine.getSpawner(new RandomFood(world.heatFloor)).xLocation == -1){
	        	g.drawString("Food spawns anywhere", tempX, tempY) ;
	        	tempY += 17 ;
	        }
	        else if(worldEngine.getSpawner(new RandomFood(world.heatFloor)).radius > 0){
	        	g.drawString("Food spawns in a sphere at", tempX, tempY) ;
	        	tempY += 17 ;
	        	g.drawString((int)(worldEngine.getSpawner(new RandomFood(world.heatFloor)).xLocation) + ", " + (int)(worldEngine.getSpawner(new RandomFood(world.heatFloor)).yLocation) + ", " + 
	        			(int)(worldEngine.getSpawner(new RandomFood(world.heatFloor)).zLocation) + ", with radius " + (int)(worldEngine.getSpawner(new RandomFood(world.heatFloor)).radius), tempX + 10, tempY) ;
	        }
	        else{
	        	g.drawString("Food spawns at the top", tempX, tempY) ;
	        	tempY += 17 ;
	        }
	        
	        tempY += 27 ;
	        g.drawString("Step actions:", tempX + 10, tempY) ;
	        tempY += 17 ;
	        g.drawString("Idle: " + (int)worldEngine.lastStepIdleCount + ", Move: " + (int)worldEngine.lastStepMoveCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Attack: " + (int)worldEngine.lastStepAttackCount + ", Detoxify: " + (int)worldEngine.lastStepDetoxCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Signal: " + (int)worldEngine.lastStepSignalCount + ", Created: " + (int)worldEngine.lastStepCreatedCount, tempX, tempY) ;
	        tempY += 22 ;
	        g.drawString("Step subconscious actions:", tempX + 10, tempY) ;
	        tempY += 17 ;
	        g.drawString("Photosynthesize: " + (int)worldEngine.lastStepPhotoCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Chemosynthesize: " + (int)worldEngine.lastStepChemoCount + ", Regenerate: " + (int)worldEngine.lastStepRegenCount + ", Relay signal: " + (int)worldEngine.lastStepSubconsciousSignalCount, tempX, tempY) ;
	        
	        tempX = 225 ;
	        tempY = 75 + worldXSize * (zoomScale + 1) ;
	        
	        g.drawString("Simulation Totals: ", tempX + 10, tempY) ;
	        tempY += 22 ;
	        g.drawString("Food added: " + (int)(worldEngine.getSpawner(new RandomFood(world.heatFloor))).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Nutrient sources added: " + (int)(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total food added: " + (int)(((worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).spawnCount + (worldEngine.getSpawner(new RandomFood(world.heatFloor))).spawnCount)), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Basic Organisms added: " + (int)(worldEngine.getSpawner(new Organism())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Advanced Organisms added: " + (int)(worldEngine.getSpawner(new AdvancedOrganism())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Cyanobacteria added: " + (int)(worldEngine.getSpawner(new Cyanobacteria())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Nitrogen Fixers added: " + (int)(worldEngine.getSpawner(new NitrogenFixer())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Phytoplankton added: " + (int)(worldEngine.getSpawner(new Phytoplankton())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Super Organisms added: " + (int)(worldEngine.getSpawner(new SuperOrganism())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Zooplankton added: " + (int)(worldEngine.getSpawner(new Zooplankton())).spawnCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total organisms added: " + (int)(((worldEngine.getSpawner(new Zooplankton())).spawnCount + (worldEngine.getSpawner(new Organism())).spawnCount + 
	        		(worldEngine.getSpawner(new AdvancedOrganism())).spawnCount + (worldEngine.getSpawner(new Cyanobacteria())).spawnCount + (worldEngine.getSpawner(new NitrogenFixer())).spawnCount +
	        		(worldEngine.getSpawner(new Phytoplankton())).spawnCount + (worldEngine.getSpawner(new SuperOrganism())).spawnCount)), tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total decays: " + (int)worldEngine.totalDecay, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total consumed: " + (int)worldEngine.totalConsumption, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total reproduction: " + (int)worldEngine.totalReproduction, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total organic energy: " + (int)worldEngine.totalEnergy, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total red nutrients: " + (int)worldEngine.totalRedNut, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total green nutrients: " + (int)worldEngine.totalGreenNut, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total blue nutrients: " + (int)worldEngine.totalBlueNut, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total deaths: " + (int)worldEngine.totalDeaths, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Deaths of old age: " + (int)worldEngine.totalNaturalCauses, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total killed: " + (int)worldEngine.totalKilled, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total poisoned: " + (int)worldEngine.totalPoisoned, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total starved: " + (int)worldEngine.totalStarved, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total malnutrition deaths: " + (int)worldEngine.totalMalnutrition, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total frozen: " + (int)worldEngine.totalFrozen, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Total cooked: " + (int)worldEngine.totalCooked, tempX, tempY) ;
	        
	        tempX += 215 ;
	        tempY -= 35 ;
	        g.drawString("Current step:", tempX + 10, tempY) ;
	        tempY += 17 ;
	        g.drawString("Organic objects: " + (int)worldEngine.organicCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Organism count: " + (int)worldEngine.organismCount, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Oldest organism: " + (int)worldEngine.oldestOrganism, tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Basic organisms: " + worldEngine.lastBasicOrganismCount , tempX, tempY) ;
	        tempX += 200 ;
	        tempY -= 52 ;
	        g.drawString("Advanced organisms: " + worldEngine.lastAdvancedOrganismCount , tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Cyanobacteria: " + worldEngine.lastCyanobacteriaCount , tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Nitrogen Fixers: " + worldEngine.lastNitrogenFixerCount , tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Phytoplankton: " + worldEngine.lastPhytoplanktonCount , tempX, tempY) ;
	        tempX += 200 ;
	        tempY -= 52 ;
	        g.drawString("Super Organisms: " + worldEngine.lastSuperOrganismCount , tempX, tempY) ;
	        tempY += 17 ;
	        g.drawString("Zooplankton: " + worldEngine.lastZooplanktonCount , tempX, tempY) ;
	        tempY += 25 ;
	        g.drawString("Most virile: " + worldEngine.mostVirile , tempX, tempY) ;
	        
	        lastGraphicsFrame = new Date().getTime() - graphicsFrameStart ;
	    }

	public void simLoop(){																			//simulation step

	    int sleepTime = 0 ;
	    while(true){
			if(bSimActive){
				resetButton.setEnabled(false) ;
				simulationStep() ;
				repaint() ;
				sleepTime = (int)(simStepDelay - worldEngine.lastWorldFrame - lastGraphicsFrame) ;
				if(sleepTime < 1)
					sleepTime = 0 ;
				try {
					Thread.sleep(sleepTime);               
				} catch(InterruptedException ex) {
					Thread.currentThread().interrupt();
				}	
			}
			else
				resetButton.setEnabled(true) ;
	    }
	}
	
	public void actionPerformed(ActionEvent event) {												//GUI buttons evaluation
		String cmd = event.getActionCommand() ;
		int tempInt = 0 ;
		Random rand = new Random() ;
		int tempX, tempY, tempZ ;
		
		String addNowChoice = (String)immediateAdd.getSelectedItem();
		String addSpawnChoice = (String)spawnerAdd.getSelectedItem();
		String driftChoice = (String)driftSelector.getSelectedItem();
		String addFromChoice = (String)addFromSelector.getSelectedItem();
		String environmentChoice = (String)environmentSelector.getSelectedItem();
		String facingChoice = (String)facingSelector.getSelectedItem();
		
		switch(cmd){
		case "ADDNOW":	switch(addNowChoice){
						case "Light":		((RandomLightSource)(worldEngine.getSpawner(new RandomLightSource())).toAdd).bVariable = false ;
											for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
											break ;
						case "Variable Light":((RandomLightSource)(worldEngine.getSpawner(new RandomLightSource())).toAdd).bVariable = true ;
												for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
													(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
											((RandomLightSource)(worldEngine.getSpawner(new RandomLightSource())).toAdd).bVariable = false ;
											break ;
						case "Heat Source":	((RandomHeatSource)(worldEngine.getSpawner(new RandomHeatSource())).toAdd).bVariable = false ;
												for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
													(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
											break ;
						case "Variable Heat Source":((RandomHeatSource)(worldEngine.getSpawner(new RandomHeatSource())).toAdd).bVariable = true ;
											for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
											((RandomHeatSource)(worldEngine.getSpawner(new RandomHeatSource())).toAdd).bVariable = false ;
											break ;
						case "Food":		for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new RandomFood(world.heatFloor))).forceAddRandom() ;
											break ;
						case "Nutrient":	for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).forceAddRandom() ;
											break ;
						case "Basic Organism":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new Organism())).forceAddRandom() ;
											break ;							
						case "Advanced Organism":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
													(worldEngine.getSpawner(new AdvancedOrganism())).forceAddRandom() ;
												break ;
						case "Cyanobacteria":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
													(worldEngine.getSpawner(new Cyanobacteria())).forceAddRandom() ;
												break ;
						case "Nitrogen Fixer":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new NitrogenFixer())).forceAddRandom() ;
											break ;							
						case "Phytoplankton":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new Phytoplankton())).forceAddRandom() ;
											break ;							
						case "Super Organism":for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new SuperOrganism())).forceAddRandom() ;
											break ;							
						case "Zooplankton" :for(int i = 0; i < Integer.parseInt(immediateQty.getText()); i++)
												(worldEngine.getSpawner(new Zooplankton())).forceAddRandom() ;
											break ;
						}
						break ;
		case "SPAWN":	switch(addSpawnChoice){
							case "Food":	(worldEngine.getSpawner(new RandomFood(world.heatFloor))).setQuantity(Integer.parseInt(spawnerQty.getText())) ;
													break ;
							case "Nutrient":	(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).setQuantity(Integer.parseInt(spawnerQty.getText())) ;
													break ;
							}
						break ;
		case "STEP":	simulationStep() ;
						repaint() ;
						break ;
		case "RESET":	world.endWorldThreads() ;
						bSimActive = false ;
						resetWorld() ;
						repaint() ;
						break ;
		case "START":	tempInt = Integer.parseInt(stepField.getText()) ;
						simStepDelay = tempInt ;
						bSimActive = true ;
						break ;
		case "STOP":	bSimActive = false ;
						break ;
		case "UP":		visualizationZPosition++ ;
						if(visualizationZPosition == world.zBounds - 1)
							goUpButton.setEnabled(false) ;
						else
							goUpButton.setEnabled(true) ;
						if(visualizationZPosition == 0)
							goDownButton.setEnabled(false) ;
						else
							goDownButton.setEnabled(true) ;
						if(!bSimActive){
							readWorld() ;
							repaint() ;
						}
						break ;
		case "DOWN":	visualizationZPosition-- ;
						if(visualizationZPosition + 1 == world.zBounds)
							goUpButton.setEnabled(false) ;
						else
							goUpButton.setEnabled(true) ;
						if(visualizationZPosition == 0)
							goDownButton.setEnabled(false) ;
						else
							goUpButton.setEnabled(true) ;
						if(!bSimActive){
							readWorld() ;
							repaint() ;
						}
						break ;
		case "XUP":		viewerX++ ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		case "XDOWN":	viewerX-- ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;		
		case "YUP":		viewerY++ ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		case "YDOWN":	viewerY-- ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		case "ZUP":		viewerZ++ ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		case "ZDOWN":	viewerZ-- ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		case "TOGGLESPAWN":	bSpawnToggle = !bSpawnToggle ;
						if(bSpawnToggle){
							if(allowBasicBox.isSelected())
								(worldEngine.getSpawner(new Organism())).setQuantity(1) ;
							else
								(worldEngine.getSpawner(new Organism())).setQuantity(0) ;
							(worldEngine.getSpawner(new AdvancedOrganism())).setQuantity(1) ;
							(worldEngine.getSpawner(new Cyanobacteria())).setQuantity(1) ;
							(worldEngine.getSpawner(new NitrogenFixer())).setQuantity(1) ;
							(worldEngine.getSpawner(new Phytoplankton())).setQuantity(1) ;
							(worldEngine.getSpawner(new SuperOrganism())).setQuantity(1) ;
							(worldEngine.getSpawner(new Zooplankton())).setQuantity(1) ;
							autoSpawnButton.setText("Stop auto-populate") ;
						}
						else{
							(worldEngine.getSpawner(new Organism())).setQuantity(0) ;
							(worldEngine.getSpawner(new AdvancedOrganism())).setQuantity(0) ;
							(worldEngine.getSpawner(new Cyanobacteria())).setQuantity(0) ;
							(worldEngine.getSpawner(new NitrogenFixer())).setQuantity(0) ;
							(worldEngine.getSpawner(new Phytoplankton())).setQuantity(0) ;
							(worldEngine.getSpawner(new SuperOrganism())).setQuantity(0) ;
							(worldEngine.getSpawner(new Zooplankton())).setQuantity(0) ;
							autoSpawnButton.setText("Start auto-populate") ;
						}
						break ;
		case "DRIFT":	switch(driftChoice){
							case "None":		world.bDrift = false ;
												world.driftVarianceX = 0 ;
												world.driftVarianceY = 0 ;
												world.driftVarianceZ = 0 ;
												world.driftSpeedX = 0 ;
												world.driftSpeedY = 0 ;
												world.driftSpeedZ = 0 ;
												break ;
							case "Low":			world.bDrift = true ;
												world.driftVarianceX = 0.25 ;
												world.driftVarianceY = 0.25 ;
												world.driftVarianceZ = 0.25 ;
												break ;
							case "High":		world.bDrift = true ;
												world.driftVarianceX = 1 ;
												world.driftVarianceY = 1 ;
												world.driftVarianceZ = 1 ;
												break ;
							case "Gravity":		world.bDrift = true ;
												world.driftSpeedZ = -4.0 ;
												break ;
							case "+X current":	world.bDrift = true ;
												world.driftSpeedX = 2.0 ;
												break ;
							case "+Y current":	world.bDrift = true ;
												world.driftSpeedY = 2.0 ;
												break ;
						}
						if(organismsDriftBox.isSelected())
							world.driftStrength = 10 ;
						else
							world.driftStrength = 2 ;
						break ;
		case "FACING":	switch(facingChoice){
							case "+X side":	world.wallBrightness[0] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[0] = Integer.parseInt(facingHeat.getText()) ;
											break ;
							case "-X side": world.wallBrightness[1] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[1] = Integer.parseInt(facingHeat.getText()) ;
											break ;
							case "+Y side":	world.wallBrightness[2] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[2] = Integer.parseInt(facingHeat.getText()) ;
											break ;
							case "-Y side":	world.wallBrightness[3] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[3] = Integer.parseInt(facingHeat.getText()) ;
											break ;
							case "+Z side":	world.wallBrightness[4] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[4] = Integer.parseInt(facingHeat.getText()) ;
											break ;
							case "-Z side":	world.wallBrightness[5] = Integer.parseInt(facingLight.getText()) ;
											world.wallTemps[5] = Integer.parseInt(facingHeat.getText()) ;
											break ;
						}
						break ;
		case "SPAWNLOCATION":	switch(addFromChoice){
									case "Anywhere":		(worldEngine.getSpawner(new RandomFood(world.heatFloor))).xLocation = -1 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).yLocation = -1 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).zLocation = -1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).xLocation = -1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).yLocation = -1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).zLocation = -1 ;
															break ;
									case "Top":				(worldEngine.getSpawner(new RandomFood(world.heatFloor))).xLocation = 0 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).yLocation = 0 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).zLocation = world.zBounds - 1 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).length = world.xBounds ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).width = world.yBounds ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).height = 1 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).radius = -1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).xLocation = 0 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).yLocation = 0 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).zLocation = world.zBounds - 1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).length = world.xBounds ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).width = world.yBounds ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).height = 1 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).radius = -1 ;
															break ;
									case "Random sphere":	tempX = rand.nextInt(world.xBounds) ;
															tempY = rand.nextInt(world.yBounds) ;
															if(world.zBounds > 0)
																tempZ = rand.nextInt(world.zBounds) ;
															else
																tempZ = 0 ;
															tempInt = rand.nextInt((world.xBounds / 6)) + (world.xBounds / 4) ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).xLocation = tempX ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).yLocation = tempY ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).zLocation = tempZ ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).length = 0 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).width = 0 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).height = 0 ;
															(worldEngine.getSpawner(new RandomFood(world.heatFloor))).radius = tempInt ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).xLocation = tempX ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).yLocation = tempY ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).zLocation = tempZ ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).length = 0 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).width = 0 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).height = 0 ;
															(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).radius = tempInt ;
															break ;
						}
						break ;
		case "ENVIRON": switch(environmentChoice){
							case "Normal":	world.heatFloor = 1 ;
											for(int i = 0; i < 6; i++)
												world.wallTemps[i] = world.heatFloor ;
											break ;
							case "Hot":		world.heatFloor = 15 ;
											for(int i = 0; i < 6; i++)
												world.wallTemps[i] = world.heatFloor ;
											break ;
							case "Cold":	world.heatFloor = -15 ;
											for(int i = 0; i < 6; i++)
												world.wallTemps[i] = world.heatFloor ;
											break ;
							case "Very cold":world.heatFloor = -40 ;
											for(int i = 0; i < 6; i++)
												world.wallTemps[i] = world.heatFloor ;
											break ;
							case "More sources": for(int i = 0; i < 4; i++)
													(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
												((RandomHeatSource)(worldEngine.getSpawner(new RandomHeatSource())).toAdd).bVariable = true ;
												for(int i = 0; i < 4; i++)
													(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
												((RandomHeatSource)(worldEngine.getSpawner(new RandomHeatSource())).toAdd).bVariable = false ;
											break ;
						}
						break ;
		case "GO":		viewerX = Integer.parseInt(entityViewX.getText()) ;
						viewerY = Integer.parseInt(entityViewY.getText()) ;
						viewerZ = Integer.parseInt(entityViewZ.getText()) ;
						entityPanelRead() ;
						if(!bSimActive)
							repaint() ;
						break ;
		}
	} 
	
	public void simulationStep(){																	//Makes everything happen
		worldEngine.worldStep() ;
		updateEntityPanel() ;
		readWorld() ;
	}
	
	public void readWorld(){																		//Read values from the appropriate Z level of the world and translate them into colors to display when redrawn
		Entity tempEnt ;
		int temp ;
		
		minLight = 256 ;
		maxLight = 0 ;
		
		for(int x = 0; x < worldXSize; x++){
			for(int y = 0; y < worldYSize; y++){
				temp = (int)Math.max(-255, Math.min(255, world.allThings[x][y][visualizationZPosition].heat)) ;
				heatMap[x][y] = new Color(Math.max(0, temp), 0, Math.max(0, temp * -1)) ;
				temp = Math.max(0, Math.min(255, world.readIllumination(x,y,visualizationZPosition))) ;
				lightMap[x][y] = new Color(temp, temp, temp / 4) ;
				if(temp > maxLight)
					maxLight = temp ;
				else if(temp < minLight)
					minLight = temp ;
				tempEnt = world.getEntity(x,y,visualizationZPosition) ;
				if(tempEnt.entityType == EntityType.ORGANIC_OBJECT || tempEnt.entityType == EntityType.ORGANISM)
					nutrientMap[x][y] = ((OrganicEntity)world.getEntity(x,y,visualizationZPosition)).nutrientHue ;
				else
					nutrientMap[x][y] = Color.BLACK ;
				
				actionMap[x][y] = Color.BLACK ;
				subconsciousActionMap[x][y] = Color.BLACK ;
				
				switch((world.getEntity(x, y, visualizationZPosition)).entityType){
				case NOTHING:		entityMap[x][y] = Color.BLACK ;
									break ;
				case LIGHT_SOURCE:	entityMap[x][y] = Color.YELLOW ;
									break ;
				case OBJECT:		entityMap[x][y] = Color.GRAY ;
									break ;
				case ORGANIC_OBJECT:entityMap[x][y] = new Color(120, 110, 40) ;
									break ;
				case ORGANISM:		entityMap[x][y] = new Color(120, 20, 140) ;
									break ;
				default:			entityMap[x][y] = Color.DARK_GRAY ;
				}
				if(world.getEntity(x, y, visualizationZPosition) instanceof AdvancedOrganism){
					if(((AdvancedOrganism)world.getEntity(x, y, visualizationZPosition)).name.equals("CYANO"))
						entityMap[x][y] = new Color(160, 60, 95) ;
					else if(((AdvancedOrganism)world.getEntity(x, y, visualizationZPosition)).name.equals("NO3FIX"))
						entityMap[x][y] = new Color(130, 155, 50) ;
					else if(((AdvancedOrganism)world.getEntity(x, y, visualizationZPosition)).name.equals("PHYTO"))
						entityMap[x][y] = new Color(30, 180, 60) ;
					else if(((AdvancedOrganism)world.getEntity(x, y, visualizationZPosition)).name.equals("SUPER"))
						entityMap[x][y] = new Color(195, 160, 60) ;
					else if(((AdvancedOrganism)world.getEntity(x, y, visualizationZPosition)).name.equals("ZOO"))
						entityMap[x][y] = new Color(40, 80, 180) ;
					else
						entityMap[x][y] = new Color(30, 130, 130) ;
					
					switch(((AdvancedOrganism)(world.getEntity(x, y, visualizationZPosition))).subconsciousAction){
					case SIGNAL:		subconsciousActionMap[x][y] = new Color(220, 190, 150) ;
										break ;
					case PHOTOSYNTHESIS:subconsciousActionMap[x][y] = new Color(20, 220, 60) ;
										break ;
					case CHEMOSYNTHESIS:subconsciousActionMap[x][y] = new Color(180, 20, 180) ;
										break ;
					case REGENERATE:	subconsciousActionMap[x][y] = new Color(100, 160, 220) ;
										break ;
					default:			break ;
					}
				}
				if(world.getEntity(x, y, visualizationZPosition) instanceof Organism){
					switch(((Organism)(world.getEntity(x, y, visualizationZPosition))).currentAction){
					case SIGNAL:		actionMap[x][y] = new Color(220, 190, 150) ;
										break ;
					case REPRODUCE:		actionMap[x][y] = new Color(20, 40, 160) ;
										break ;
					case DIE:			actionMap[x][y] = new Color(100, 100, 20) ;
										break ;
					case MOVE:			actionMap[x][y] = new Color(30, 160, 150) ;
										break ;
					case IDLE:			actionMap[x][y] = Color.GRAY ;
										break ;
					case CREATED:		actionMap[x][y] = Color.CYAN ;
										break ;
					case EAT:			actionMap[x][y] = new Color(180, 180, 50) ;
										break ;
					case ATTACK:		actionMap[x][y] = new Color(220, 40, 40) ;
										break ;
					case DETOX:			actionMap[x][y] = new Color(110, 20, 130) ;
										break ;
					default:			break ;
					}
				}
			}
		}
	}
	
	public void resetWorld(){																								//Generate a new world and world engine
		visualizationZPosition = 0 ;
		if(bigWorldBox.isSelected()){
			bLargeWorld = true ;
			worldXSize = 100 ;
			worldYSize = 100 ;
			zoomScale = 3 ;
		}
		else {
			bLargeWorld = false ;
			worldXSize = 50 ;
			worldYSize = 50 ;
			zoomScale = 7 ;
		}
		if(world != null)
			world.endWorldThreads();
		worldZSize = Integer.parseInt((String)depthSelector.getSelectedItem()) ;
		
		world = new World(worldXSize, worldYSize, worldZSize, 1, 0, 3.0, 1.0) ;
		worldEngine = new WorldEngine(world) ;
		
		heatMap = new Color[worldXSize][worldYSize] ;
		lightMap = new Color[worldXSize][worldYSize] ;
		nutrientMap = new Color[worldXSize][worldYSize] ;
		entityMap = new Color[worldXSize][worldYSize] ;
		actionMap = new Color[worldXSize][worldYSize] ;
		subconsciousActionMap = new Color[worldXSize][worldYSize] ;
		
		worldEngine.addSpawner(new SpawnRandomizer(world, new RandomLightSource(), 0, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new RandomHeatSource(), 0, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new RandomFood(world.heatFloor), 0, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new RandomNutrientSource(world.heatFloor, true), 0, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new Organism(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new AdvancedOrganism(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new Cyanobacteria(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new NitrogenFixer(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new Phytoplankton(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new SuperOrganism(), true, 20, 200, 0));
		worldEngine.addSpawner(new SpawnRandomizer(world, new Zooplankton(), true, 20, 200, 0));
		
		String worldTemplateChoice = (String)resetTemplate.getSelectedItem();
		switch(worldTemplateChoice){
		case "Nothing":	break ;
		case "Low":		for(int i = 0; i < 2; i++)
							(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
						(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
						(worldEngine.getSpawner(new RandomFood(world.heatFloor))).setQuantity(1) ;
						break ;
		case "Medium":	for(int i = 0; i < 4; i++)
							(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
						for(int i = 0; i < 2; i++)
							(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
						(worldEngine.getSpawner(new RandomFood(world.heatFloor))).setQuantity(1) ;
						(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).setQuantity(2) ;
						break ;
		case "High":	for(int i = 0; i < 7; i++)
							(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
						for(int i = 0; i < 4; i++)
							(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
						(worldEngine.getSpawner(new RandomFood(world.heatFloor))).setQuantity(4) ;
						(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).setQuantity(4) ;
						break ;
		case"Bountiful":for(int i = 0; i < 10; i++)
							(worldEngine.getSpawner(new RandomLightSource())).forceAddRandom() ;
						for(int i = 0; i < 6; i++)
							(worldEngine.getSpawner(new RandomHeatSource())).forceAddRandom() ;
						(worldEngine.getSpawner(new RandomFood(world.heatFloor))).setQuantity(8) ;
						(worldEngine.getSpawner(new RandomNutrientSource(world.heatFloor, true))).setQuantity(9) ;
						break ;
		}
		
		if(visualizationZPosition + 1 == world.zBounds)
			goUpButton.setEnabled(false) ;
		else
			goUpButton.setEnabled(true) ;
		if(visualizationZPosition == 0)
			goDownButton.setEnabled(false) ;
		else
			goUpButton.setEnabled(true) ;
		
		worldEngine.worldStep() ;
		readWorld() ;
		entityPanelRead() ;
	}
	
	public Color gammaUp(Color toUp, double factor){											//A gamma correction method for making the light/heat visualization more visually meaningful
		int red, green, blue ;
		double gammaFactor = 1 / factor ;
		red = toUp.getRed();
		green = toUp.getGreen();
		blue = toUp.getBlue();
		
		red = (int) (255 * (Math.pow((double) red / (double) 255, gammaFactor)));
        green = (int) (255 * (Math.pow((double) green / (double) 255, gammaFactor)));
        blue = (int) (255 * (Math.pow((double) blue / (double) 255, gammaFactor)));
        
        return new Color(red, green, blue) ;
	}
	
	public void entityPanelRead(){																//Update the location of the cursor
		if(viewerX >= world.xBounds - 1){
			viewerX = world.xBounds - 1 ;
			entityViewXUp.setEnabled(false) ;
		}
		else
			entityViewXUp.setEnabled(true) ;		
		if(viewerX <= 0){
			viewerX = 0 ;
			entityViewXDown.setEnabled(false) ;
		}
		else
			entityViewXDown.setEnabled(true) ;	
		
		if(viewerY >= world.yBounds - 1){
			viewerY = world.yBounds - 1 ;
			entityViewYUp.setEnabled(false) ;
		}
		else
			entityViewYUp.setEnabled(true) ;		
		if(viewerY <= 0){
			viewerY = 0 ;
			entityViewYDown.setEnabled(false) ;
		}
		else
			entityViewYDown.setEnabled(true) ;	
		
		if(viewerZ >= world.zBounds - 1){
			viewerZ = world.zBounds - 1 ;
			entityViewZUp.setEnabled(false) ;
		}
		else
			entityViewZUp.setEnabled(true) ;		
		if(viewerZ <= 0){
			viewerZ = 0 ;
			entityViewZDown.setEnabled(false) ;
		}
		else
			entityViewZDown.setEnabled(true) ;	
		
		entityViewX.setText(Integer.toString(viewerX)) ;
		entityViewY.setText(Integer.toString(viewerY)) ;
		entityViewZ.setText(Integer.toString(viewerZ)) ;
		
		updateEntityPanel() ;
	}
	
	public void updateEntityPanel(){																			//Update the entityViewer with current data
		DecimalFormat df = new DecimalFormat("#.##") ;
		String abilitiesText = null ;
		String abilitiesText2 = null ;
		
		entityPanelViewed = world.getEntity(viewerX,  viewerY,  viewerZ) ;
		
        entityViewType.setText(entityPanelViewed.getClass().getName()) ;
        if(entityPanelViewed instanceof AdvancedOrganism){
        	entityViewName.setText(((AdvancedOrganism)entityPanelViewed).name + " " + (int)((AdvancedOrganism)entityPanelViewed).id + ", generation " + (int)((AdvancedOrganism)entityPanelViewed).generation) ;    
	        entityViewHP.setText("Health points: " + ((AdvancedOrganism)entityPanelViewed).healthPoints) ;
	        entityViewAction.setText("Action: " + ((Organism)entityPanelViewed).currentAction) ;
	        entityViewSubAction.setText("Subconscious action: " + ((Organism)entityPanelViewed).subconsciousAction) ;
	        
	        if(!((AdvancedOrganism)entityPanelViewed).bSessile)
	        	abilitiesText = "Can move" ;
	        if(((AdvancedOrganism)entityPanelViewed).bPhotosynthetic)
	        	abilitiesText2 = "Can photosynthesize" ;
	        if(((AdvancedOrganism)entityPanelViewed).bChemotrophic){
	        	if(abilitiesText2 != null)
	        		abilitiesText2 += ", chemosynthesize" ;
	        	else
	        		abilitiesText2 = "Can chemosynthesize" ;
	        }
	        if(((AdvancedOrganism)entityPanelViewed).bCanSignal){
	        	if(abilitiesText != null)
	        		abilitiesText += ", signal" ;
	        	else
	        		abilitiesText = "Can signal" ;
	        }
	        if(((AdvancedOrganism)entityPanelViewed).bCanAttack){
	        	if(abilitiesText != null)
	        		abilitiesText += ", attack" ;
	        	else
	        		abilitiesText = "Can attack" ;
	        }
	        
	        if(abilitiesText == null)
	        	abilitiesText = "" ;
	        if(abilitiesText2 == null)
	        	abilitiesText2 = "" ;
	        
	        entityViewAbilities.setText(abilitiesText) ;
	        entityViewAbilities2.setText(abilitiesText2) ;
	        if(((AdvancedOrganism)entityPanelViewed).progeny > 0)
	        	entityViewProgeny.setText("Has " + (int)((AdvancedOrganism)entityPanelViewed).progeny + " offspring") ;
	        else
	        	entityViewProgeny.setText("Has no offspring") ;
	        if(((AdvancedOrganism)entityPanelViewed).attacker != null)
	        	entityViewAttacker.setText("Was attacked by " + ((AdvancedOrganism)entityPanelViewed).attacker.name + " " + ((AdvancedOrganism)entityPanelViewed).attacker.id + "." + ((AdvancedOrganism)entityPanelViewed).attacker.generation) ;
	        else
	        	entityViewAttacker.setText("Has not been recently attacked") ;
	        if(((AdvancedOrganism)entityPanelViewed).recievedSignal.message != ' ')
	        	entityViewSignalRecieved.setText("Last recieved: " + ((AdvancedOrganism)entityPanelViewed).recievedSignal.message + " at strength " + ((AdvancedOrganism)entityPanelViewed).recievedSignal.strength + " from " + ((AdvancedOrganism)entityPanelViewed).recievedSignal.sender) ;
	        else
	        	entityViewSignalRecieved.setText("Has not been recently signaled") ;
        }
        else{
        	if(entityPanelViewed instanceof Organism){
        		entityViewName.setText("Unnamed Basic Organism") ;    
    	        entityViewHP.setText("Health points: 1") ;
    	        entityViewAction.setText("Action: " + ((Organism)entityPanelViewed).currentAction) ;
    	        entityViewSubAction.setText("Subconscious action: " + ((Organism)entityPanelViewed).subconsciousAction) ;
    	        entityViewAbilities.setText("Can photosynthesize") ;
    	        entityViewProgeny.setText("Has an unknown number of offspring") ;
    	        entityViewAttacker.setText("Dies if attacked") ;
    	        entityViewSignalRecieved.setText("Cannot recieve signals") ;
        	}
        	else{
        		entityViewName.setText("No name") ;    
    	        entityViewHP.setText("No health points") ;
    	        entityViewAction.setText("No action") ;
    	        entityViewSubAction.setText("No subconscious action") ;
    	        entityViewAbilities.setText("No abilities") ;
    	        entityViewProgeny.setText("Cannot reproduce") ;
    	        entityViewAttacker.setText("Cannot be attacked") ;
    	        entityViewSignalRecieved.setText("Cannot recieve signals") ;
        	}
        }
        
        if(entityPanelViewed instanceof OrganicEntity){
        	entityViewEnergy.setText("Energy: " + (int)((OrganicEntity)entityPanelViewed).energy + ", Toxicity: " + (int)((OrganicEntity)entityPanelViewed).toxicity) ;
            entityViewNutrients.setText("Nutrients - Red: " + (int)((OrganicEntity)entityPanelViewed).nutrientHue.getRed() + ", Green: " + 
            		(int)((OrganicEntity)entityPanelViewed).nutrientHue.getGreen() + ", Blue: " + (int)((OrganicEntity)entityPanelViewed).nutrientHue.getBlue()) ;
        }
        else{
        	entityViewEnergy.setText("No organic energy") ;
            entityViewNutrients.setText("No nutrients") ;
        }
        
        entityViewHeat.setText("Heat: " + df.format(entityPanelViewed.heat)) ;
        entityViewLight.setText("Illumination: " + (int)entityPanelViewed.illumination) ;
        entityViewLifetime.setText("Age: " + (int)entityPanelViewed.lifetime) ;
	}
	
	public void autoGamma(){																		//Gamma correction for heat/light maps
		if(minLight > 40)
			lightAutoGamma = 1.0 ;
		else if(minLight > 30)
			lightAutoGamma = 1.5 ;
		else if(minLight > 16)
			lightAutoGamma = 2.5 ;
		else if(minLight > 7)
			lightAutoGamma = 3.0 ;
		else
			lightAutoGamma = 3.5 ;
		
		if(worldEngine.stepMinHeat > 40)
			heatAutoGamma = 0.5 ;
		else if(worldEngine.stepMinHeat > 25)
			heatAutoGamma = 1.0 ;
		else if(worldEngine.stepMinHeat > 15)
			heatAutoGamma = 2.1 ;
		else if(worldEngine.stepMinHeat > 6)
			heatAutoGamma = 3.3 ;
		else
			heatAutoGamma = 4.5 ;		
	}
	
	public void attachShutDownHook(){																//Called to end threads, haven't noticed any difference from before this was added.
		Runtime.getRuntime().addShutdownHook(new Thread() {
		@Override
		public void run() {
			world.endWorldThreads() ;
		    }
		});
	}
}
